<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Cms\Model\ResourceModel\Block\CollectionFactory as BlockCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class CmsBlock
 * @package FiloBlu\Refilo\Model\Exporter
 */
class CmsBlock extends AbstractExporter
{
    /**
     * @var BlockCollectionFactory
     */
    protected $blockCollectionFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param BlockCollectionFactory $blockCollectionFactory
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        BlockCollectionFactory $blockCollectionFactory
    )
    {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
        $this->storeManager = $storeManager;
        $this->blockCollectionFactory = $blockCollectionFactory;
    }


    public function preExecute()
    {
        foreach ($this->getCollections() as $collection) {
            $this->getConnection()->deleteDocumentsByProperties(
                $this->getDatabase(),
                $collection,
                ['metacode' => 'block']
            );
        }
    }

    /**
     * @return \Generator
     */
    public function getCollections() : iterable
    {
        foreach ($this->storeManager->getStores() as $store) {
            yield "contents_{$store->getCode()}";
        }
    }

    /**
     *
     */
    public function execute()
    {
        foreach ($this->storeManager->getStores() as $key => $value) {

            $blocks = $this->blockCollectionFactory->create()
                ->addStoreFilter($value)
                ->addFilter('is_active', 1);

            $bulk = [];
            foreach ($blocks as $item) {
                /* $block = new \stdClass();
                 $block->_id = "block_" . $item->getId() . "_" . $value->getData('code');
                 $block->metacode = 'block';
                 $block->store = $value->getData('code');
                 $block->content_id = (int)  $item->getId();
                 $block->url_key = $item->getIdentifier();
                 $block->idx = $item->getIdentifier();
                 $block->title = $item->getTitle();
                 $block->type = 'static_block';
                 $block->status = $item->isActive();
                 $block->content = $item->getContent();
                 $block->display_entity = $item->getDisplayOn();
                 $block->display_selection_type = $item->getSelectionType();
                 $block->display_selection_target = $this->parseTargetSelection($item->getSelectionTarget());
                 $block->display_container = $item->getRefiloLayoutContainer();
                 $block->display_sort_order = (int)$item->getRefiloLayoutOrder();
                 $block->listing_display_page = (int)$item->getListingDisplayPage();
                 $block->listing_display_position = (int)$item->getListingDisplayPosition();
                 $block->listing_columns_spacing = (int)$item->getListingColumnsSpacing();
                 $block->listing_row_spacing = (int)$item->getListingRowSpacing();*/

                $bulk[] = [
                    '_id'                      => "block_" . $item->getId() . "_" . $value->getData('code'),
                    'metacode'                 => 'block',
                    'store'                    => $value->getData('code'),
                    'content_id'               => (int)$item->getId(),
                    'url_key'                  => $item->getIdentifier(),
                    'idx'                      => $item->getIdentifier(),
                    'title'                    => $item->getTitle(),
                    'type'                     => 'static_block',
                    'status'                   => $item->isActive(),
                    'content'                  => $item->getContent(),
                    'display_entity'           => $item->getDisplayOn(),
                    'display_selection_type'   => $item->getSelectionType(),
                    'display_selection_target' => $this->parseTargetSelection($item->getSelectionTarget()),
                    'display_container'        => $item->getRefiloLayoutContainer(),
                    'display_sort_order'       => (int)$item->getRefiloLayoutOrder(),
                    'listing_display_page'     => (int)$item->getListingDisplayPage(),
                    'listing_display_position' => (int)$item->getListingDisplayPosition(),
                    'listing_columns_spacing'  => (int)$item->getListingColumnsSpacing(),
                    'listing_row_spacing'      => (int)$item->getListingRowSpacing()
                ];
            }

            try {
                $this->getConnection()->update(
                    '_id',
                    $bulk,
                    "contents_" . $value->getCode(),
                    true,
                    $this->getDatabase()
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $value
     * @return false|string[]
     */
    protected function parseTargetSelection($value)
    {
        $strippedValue = preg_replace('/\s/', '', $value);
        return explode(',', $strippedValue);
    }
}
