<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use LogicException;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\ResourceModel\Eav\Attribute;
use Magento\Eav\Api\AttributeRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Swatches\Helper\Data;
use Magento\Swatches\Helper\Media;
use Magento\Swatches\Model\Swatch;

/**
 * Class AttributeProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class ProductAttributeProvider extends BaseProvider
{
    /**
     * @var \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper
     */
    protected $productAttributeHelper;

    /** @var StoreInterface|null */
    protected $store;

    /**
     * ProductAttributeProvider constructor.
     * @param \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper $productAttributeHelper
     */
    public function __construct(
        \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper $productAttributeHelper
    ) {
        $this->productAttributeHelper = $productAttributeHelper;
    }

    /**
     *
     */
    public function rewind()
    {
       $attributes = $this->productAttributeHelper->getAttributes()[$this->store->getId()];

       foreach ($attributes as $code => $data)
       {
           $attribute = new \FiloBlu\Refilo\Remote\Entity\Attribute();
           $attribute->setId($code);
           $attribute->addData($data);
           $this->items[$code] = $attribute;
       }

        if (!$this->hasReadHandler()) {
            return;
        }

        $this->getReadHandler()->onRead($this->items, count($attributes), $this->getReadHandlerArguments());
    }

    /**
     * @param StoreInterface $store
     * @return mixed
     */
    public function getAttributes(StoreInterface $store)
    {
        $this->init();
        return $this->attributes[$store->getId()];
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store;
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        throw new LogicException('Can not filter by website');
    }
}
